@@ -1,7 +1,7 @@
 # SPDX-License-Identifier: MIT
 # Copyright (c) 2022 MBition GmbH
 
-from typing import Dict, Iterable, Optional, OrderedDict, Union
+from typing import Any, List, Dict, Iterable, Optional, OrderedDict, Union
 
 from .dataobjectproperty import DataObjectProperty, DopBase
 from .decodestate import DecodeState, ParameterValuePair
@@ -13,17 +13,16 @@ from .parameters import Parameter, ParameterWithDOP, read_parameter_from_odx
 from .parameters import CodedConstParameter, MatchingRequestParameter, ValueParameter
 from .utils import read_description_from_odx
 
-
 class BasicStructure(DopBase):
     def __init__(self,
                  id,
                  short_name,
-                 parameters: Iterable[Parameter],
+                 parameters: Iterable[Union[Parameter, "EndOfPduField"]], # type: ignore
                  long_name=None,
                  byte_size=None,
                  description=None):
         super().__init__(id, short_name, long_name=long_name, description=description)
-        self.parameters = NamedItemList(lambda par: par.short_name, parameters)
+        self.parameters : NamedItemList[Union[Parameter, "EndOfPduField"]] = NamedItemList(lambda par: par.short_name, parameters) # type: ignore
 
         self._byte_size = byte_size
 
@@ -51,8 +50,43 @@ class BasicStructure(DopBase):
                 break
         return prefix
 
-    def get_required_parameters(self):
-        return filter(lambda p: p.is_required(), self.parameters)
+    def get_required_parameters(self) -> List[Parameter]:
+        """Return the list of parameters which are required for
+        encoding the structure."""
+        return [p for p in self.parameters if p.is_required()]
+
+    def get_free_parameters(self) -> List[Union[Parameter, "EndOfPduField"]]: # type: ignore
+        """Return the list of parameters which can be freely specified by
+        the user when encoding the structure.
+
+        This means all required parameters plus the parameters that
+        can be omitted minus those which are implicitly specified by
+        the corresponding request (in the case of responses).
+
+        """
+        from .endofpdufield import EndOfPduField
+
+        result : List[Union[Parameter, EndOfPduField]] = []
+        for param in self.parameters:
+            if isinstance(param, EndOfPduField):
+                result.append(param)
+                continue
+            elif not param.is_required():
+                continue
+            # The user cannot specify MatchingRequestParameters freely!
+            elif isinstance(param, MatchingRequestParameter):
+                continue
+            result.append(param)
+
+        return result
+
+    def free_parameters_info(self) -> str:
+        """Return a human readable description of the structure's
+        free parameters.
+        """
+        from .parameter_info import parameter_info
+
+        return parameter_info(self.get_free_parameters())
 
     def convert_physical_to_internal(self,
                                      param_values: dict,
